@@ -18,6 +18,7 @@ import tg.i18n
 from tg import TGController, expose, response, request, abort
 from tests.base import TestWSGIController, make_app, setup_session_dir, teardown_session_dir, create_request
 from webtest import TestApp
+from tg.renderers.base import RendererFactory
 
 from tg.wsgiapp import TGApp
 from tg._compat import PY3
@@ -193,12 +194,6 @@ class TestAppConfig:
         #This is here to avoid that other tests keep using the forced controller
         config.pop('tg.root_controller')
 
-    def test_amf_initialization(self):
-        conf = AppConfig(minimal=True)
-        conf.renderers.append('amf')
-        app = conf.make_wsgi_app()
-        assert 'amf' not in conf.renderers
-
     def test_enable_routes(self):
         if PY3: raise SkipTest()
 
@@ -235,14 +230,24 @@ class TestAppConfig:
         self.config.call_on_shutdown = ['not callable']
         self.config._setup_startup_and_shutdown()
 
-    @raises(AtExitTestException)
     def test_setup_startup_and_shutdown_shutdown_callable(self):
         def func():
             raise AtExitTestException()
 
-        self.config.call_on_shutdown = [func]
-        self.config._setup_startup_and_shutdown()
-        atexit._run_exitfuncs()
+        _registered_exit_funcs = []
+        def _fake_atexit_register(what):
+            _registered_exit_funcs.append(what)
+
+        _real_register = atexit.register
+        atexit.register = _fake_atexit_register
+
+        try:
+            self.config.call_on_shutdown = [func]
+            self.config._setup_startup_and_shutdown()
+        finally:
+            atexit.register = _real_register
+
+        assert func in _registered_exit_funcs, _registered_exit_funcs
 
     def test_setup_helpers_and_globals(self):
         self.config.setup_helpers_and_globals()
@@ -259,30 +264,6 @@ class TestAppConfig:
 
         assert len(ConfigWithSetupAuthBackend.called) >= 1
 
-    def test_setup_chameleon_genshi_renderer(self):
-        if PY3: raise SkipTest()
-
-        self.config.paths.templates = 'template_path'
-        self.config.setup_chameleon_genshi_renderer()
-
-    def test_setup_kajiki_renderer(self):
-        if PY3: raise SkipTest()
-
-        self.config.paths.templates = 'template_path'
-        self.config.setup_kajiki_renderer()
-
-    def test_setup_genshi_renderer(self):
-        self.config.paths.templates = 'template_path'
-        self.config.setup_genshi_renderer()
-
-    def test_setup_jinja_renderer(self):
-        self.config.paths.templates = 'template_path'
-        self.config.setup_jinja_renderer()
-
-    def test_setup_mako_renderer(self):
-        self.config.paths.templates = ['template_path']
-        self.config.setup_mako_renderer(use_dotted_templatenames=True)
-
     def test_setup_jinja_without_package(self):
         class RootController(TGController):
             @expose()
@@ -604,12 +585,16 @@ class TestAppConfig:
             pass
 
         milestones.config_ready._reset()
+        milestones.environment_loaded._reset()
+ 
         self.config.register_hook('startup', dummy)
         self.config.register_hook('shutdown', dummy)
         self.config.register_hook('controller_wrapper', dummy)
         for hook_name in self.config.hooks.keys():
             self.config.register_hook(hook_name, dummy)
+
         milestones.config_ready.reach()
+        milestones.environment_loaded.reach()
 
         for hooks in self.config.hooks.values():
             assert hooks
@@ -1161,13 +1146,13 @@ class TestAppConfig:
         tg.hooks.register('controller_wrapper', None)
 
     @raises(TGConfigError)
-    def test_global_controller_wrapper_after_config_ready(self):
-        milestones.config_ready.reach()
+    def test_global_controller_wrapper_after_milestone_reached(self):
+        milestones.environment_loaded.reach()
         tg.hooks.wrap_controller(None)
 
     @raises(TGConfigError)
-    def test_dedicated_controller_wrapper_after_config_ready(self):
-        milestones.config_ready.reach()
+    def test_dedicated_controller_wrapper_after_milestone_reached(self):
+        milestones.environment_loaded.reach()
 
         def f():
             pass
@@ -1328,3 +1313,161 @@ class TestAppConfig:
 
         resp = app.get('/test', status=403)
         assert 'ERROR!!!' in resp, resp
+
+    def test_errorware_configuration(self):
+        class RootController(TGController):
+            @expose()
+            def test(self, *args, **kwargs):
+                return 'HI'
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        app = conf.make_wsgi_app(global_conf={'trace_errors.error_email': 'test@domain.com'},
+                                 full_stack=True)
+        app = TestApp(app)
+
+        resp = app.get('/test')
+        assert 'HI' in resp, resp
+
+        assert config['tg.errorware']['error_email'] == 'test@domain.com'
+        assert config['tg.errorware']['error_subject_prefix'] == 'WebApp Error: '
+        assert config['tg.errorware']['error_message'] == 'An internal server error occurred'
+
+    def test_tw2_unsupported_renderer(self):
+        import tw2.core
+
+        class RootController(TGController):
+            @expose()
+            def test(self, *args, **kwargs):
+                rl = tw2.core.core.request_local()
+                tw2conf = rl['middleware'].config
+                return ','.join(tw2conf.preferred_rendering_engines)
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        conf.prefer_toscawidgets2 = True
+        conf.renderers = ['kajiki', 'genshi']
+        conf.default_renderer = 'kajiki'
+
+        app = conf.make_wsgi_app(full_stack=True)
+        app = TestApp(app)
+
+        resp = app.get('/test')
+        assert 'genshi' in resp, resp
+
+    def test_tw2_renderers_preference(self):
+        import tw2.core
+
+        class RootController(TGController):
+            @expose()
+            def test(self, *args, **kwargs):
+                rl = tw2.core.core.request_local()
+                tw2conf = rl['middleware'].config
+                return ','.join(tw2conf.preferred_rendering_engines)
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        conf.prefer_toscawidgets2 = True
+        conf.renderers = ['genshi']
+        conf.default_renderer = 'genshi'
+
+        app = conf.make_wsgi_app(full_stack=True)
+        app = TestApp(app)
+
+        resp = app.get('/test')
+        assert 'genshi' in resp, resp
+
+    def test_tw2_unsupported(self):
+        import tw2.core
+
+        class RootController(TGController):
+            @expose()
+            def test(self, *args, **kwargs):
+                rl = tw2.core.core.request_local()
+                tw2conf = rl['middleware'].config
+                return ','.join(tw2conf.preferred_rendering_engines)
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        conf.prefer_toscawidgets2 = True
+        conf.renderers = ['kajiki']
+        conf.default_renderer = 'kajiki'
+
+        try:
+            app = conf.make_wsgi_app(full_stack=True)
+            assert False
+        except TGConfigError as e:
+            assert 'None of the configured rendering engines is supported' in str(e)
+
+    def test_backward_compatible_engine_failed_setup(self):
+        class RootController(TGController):
+            @expose()
+            def test(self, *args, **kwargs):
+                return 'HELLO'
+
+        def setup_broken_renderer():
+            return False
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        conf.setup_broken_renderer = setup_broken_renderer
+        conf.renderers = ['json', 'broken']
+
+        app = conf.make_wsgi_app(full_stack=True)
+        assert conf.renderers == ['json']
+
+    def test_backward_compatible_engine_success_setup(self):
+        class RootController(TGController):
+            @expose()
+            def test(self, *args, **kwargs):
+                return 'HELLO'
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+
+        def setup_broken_renderer():
+            conf.render_functions.broken = 'BROKEN'
+            return True
+
+        conf.setup_broken_renderer = setup_broken_renderer
+        conf.renderers = ['json', 'broken']
+
+        app = conf.make_wsgi_app(full_stack=True)
+        assert conf.renderers == ['json', 'broken']
+        assert conf.render_functions.broken == 'BROKEN'
+
+    def test_render_factory_success(self):
+        class RootController(TGController):
+            @expose()
+            def test(self, *args, **kwargs):
+                return 'HELLO'
+
+        class FailedFactory(RendererFactory):
+            engines = {'broken': {'content_type': 'text/plain'}}
+
+            @classmethod
+            def create(cls, config, app_globals):
+                return {'broken': 'BROKEN'}
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        conf.register_rendering_engine(FailedFactory)
+        conf.renderers = ['json', 'broken']
+
+        app = conf.make_wsgi_app(full_stack=True)
+        assert conf.renderers == ['json', 'broken']
+        assert conf.render_functions.broken == 'BROKEN'
+
+    def test_render_factory_failure(self):
+        class RootController(TGController):
+            @expose()
+            def test(self, *args, **kwargs):
+                return 'HELLO'
+
+        class FailedFactory(RendererFactory):
+            engines = {'broken': {'content_type': 'text/plain'}}
+
+            @classmethod
+            def create(cls, config, app_globals):
+                return None
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        conf.register_rendering_engine(FailedFactory)
+        conf.renderers = ['json', 'broken']
+
+        app = conf.make_wsgi_app(full_stack=True)
+        assert conf.renderers == ['json']
+
