@@ -1,12 +1,11 @@
 # -*- coding: utf-8 -*-
-
-import pylons
 import tg
 from tg.controllers import *
 from tg.exceptions import HTTPFound
 from nose.tools import eq_
 from tests.base import TestWSGIController, make_app, setup_session_dir, teardown_session_dir, create_request
 from tg.util import no_warn
+from tg._compat import u_, string_type
 
 def setup():
     setup_session_dir()
@@ -34,7 +33,7 @@ def test_lowerapproots():
 @no_warn
 def test_multi_values():
     create_request('/')
-    r = url("/foo", params=dict(bar=(u"asdf",u"qwer")))
+    r = url("/foo", params=dict(bar=("asdf", "qwer")))
     assert r in \
             ["/foo?bar=qwer&bar=asdf", "/foo?bar=asdf&bar=qwer"], r
     r = url("/foo", params=dict(bar=[1,2]))
@@ -45,20 +44,15 @@ def test_multi_values():
 def test_unicode():
     """url() can handle unicode parameters"""
     create_request("/")
-    unicodestring = (u'\N{LATIN SMALL LETTER A WITH GRAVE}'
-        u'\N{LATIN SMALL LETTER E WITH GRAVE}'
-        u'\N{LATIN SMALL LETTER I WITH GRAVE}'
-        u'\N{LATIN SMALL LETTER O WITH GRAVE}'
-        u'\N{LATIN SMALL LETTER U WITH GRAVE}')
+    unicodestring =  u_('àèìòù')
     eq_(url('/', params=dict(x=unicodestring)),
         '/?x=%C3%A0%C3%A8%C3%AC%C3%B2%C3%B9'
         )
-
 @no_warn
 def test_list():
     """url() can handle list parameters, with unicode too"""
     create_request("/")
-    value = url('/', params=dict(foo=['bar', u'\N{LATIN SMALL LETTER A WITH GRAVE}'])),
+    value = url('/', params=dict(foo=['bar', u_('à')])),
     assert '/?foo=bar&foo=%C3%A0' in value, value
 
 @no_warn
@@ -82,7 +76,42 @@ def test_lurl():
     params = {'spamm':'eggs', 'hamm':None }
     assert url('/foo', params=params) == str(lurl('/foo', params=params))
 
-#def test_approotsWithPath():
-#    create_request('/coolsite/root/subthing/', {'SCRIPT_NAME' : '/subthing'})
-#    pylons.config.update({"server.webpath":"/coolsite/root"})
-#    eq_("/coolsite/root/subthing/foo", pylons.url("/foo"))
+@no_warn
+def test_url_qualified():
+    """url() can handle list parameters, with unicode too"""
+    create_request("/")
+    value = url('/', qualified=True)
+    assert value.startswith('http')
+
+@no_warn
+def test_lurl():
+    """url() can handle list parameters, with unicode too"""
+    create_request("/")
+    value = lurl('/lurl')
+    assert not isinstance(value, string_type)
+    assert value.startswith('/lurl')
+    assert str(value) == repr(value) == value.id == value.encode('utf-8').decode('utf-8') == value.__html__()
+
+@no_warn
+def test_lurl_format():
+    """url() can handle list parameters, with unicode too"""
+    create_request("/")
+    value = lurl('/lurl/{0}')
+    value = value.format('suburl')
+    assert value == '/lurl/suburl', value
+
+@no_warn
+def test_lurl_add():
+    """url() can handle list parameters, with unicode too"""
+    create_request("/")
+    value = lurl('/lurl')
+    value = value + '/suburl'
+    assert value == '/lurl/suburl', value
+
+@no_warn
+def test_lurl_radd():
+    """url() can handle list parameters, with unicode too"""
+    create_request("/")
+    value = lurl('/lurl')
+    value = '/suburl' + value
+    assert value == '/suburl/lurl', value
