@@ -1,97 +1,57 @@
 # -*- coding: utf-8 -*-
+from functools import partial
 from nose.tools import raises
 from nose import SkipTest
+from crank.util import get_params_with_argspec
 
 import tg
 import tests
 from json import loads, dumps
 
-from tg.controllers import TGController
-from tg.decorators import expose, validate, before_render
+from tg.controllers import TGController, DecoratedController
+from tg.decorators import expose, validate, before_render, before_call
 from tests.base import (TestWSGIController, data_dir,
     make_app, setup_session_dir, teardown_session_dir)
 
-from tg._compat import PY3, unicode_text, u_
-from tg.validation import TGValidationError
+from tg._compat import PY3, unicode_text, u_, default_im_func
+from tg.util import call_controller
+from tg.validation import TGValidationError, validation_errors
+
+from formencode import validators, Schema
+
+import tw2.core as tw2c
+import tw2.forms as tw2f
+
+class MovieForm(tw2f.TableForm):
+    title = tw2f.TextField(validator=tw2c.Required)
+    year = tw2f.TextField(size=4, validator=tw2c.IntValidator)
+movie_form = MovieForm(action='save_movie')
+
+class Pwd(Schema):
+    pwd1 = validators.String(not_empty=True)
+    pwd2 = validators.String(not_empty=True)
+    chained_validators = [validators.FieldsMatch('pwd1', 'pwd2')]
+
+class FormWithFieldSet(tw2f.TableForm):
+    class fields1(tw2f.ListFieldSet):
+        f1 = tw2f.TextField(validator=tw2c.Required)
+
+    class fields2(tw2f.ListFieldSet):
+        f2 = tw2f.TextField(validator=tw2c.IntValidator)
 
 if not PY3:
-    from formencode import validators, Schema
-
     from tw.forms import TableForm, TextField
     from tw.api import WidgetsList
 
-    import tw2.core as tw2c
-    import tw2.forms as tw2f
-
-    class MovieForm(tw2f.TableForm):
-        title = tw2f.TextField(validator=tw2c.Required)
-        year = tw2f.TextField(size=4, validator=tw2c.IntValidator)
-    movie_form = MovieForm(action='save_movie')
-
     class MyForm(TableForm):
         class fields(WidgetsList):
             """This WidgetsList is just a container."""
             title=TextField(validator = validators.NotEmpty())
             year = TextField(size=4, validator=validators.Int())
     myform = MyForm("my_form", action='create')
-
-    class Pwd(Schema):
-        pwd1 = validators.String(not_empty=True)
-        pwd2 = validators.String(not_empty=True)
-        chained_validators = [validators.FieldsMatch('pwd1', 'pwd2')]
 else:
-    movie_form = None
     myform = None
 
-    class validators(object):
-        """Simulate Formencode"""
-        Invalid = TGValidationError
-
-        class FancyValidator(object):
-            def _to_python(self, value):
-                return value
-
-            def validate_python(self, value, state=None):
-                if not value:
-                    raise TGValidationError('Empty')
-
-            def to_python(self, value, state=None):
-                try:
-                    pyv = self._to_python(value)
-                except Exception as e:
-                    raise TGValidationError(str(e))
-
-                self.validate_python(pyv, None)
-                return pyv
-
-        class Int(FancyValidator):
-            def _to_python(self, value):
-                try:
-                    return int(value)
-                except:
-                    raise TGValidationError('Must be an integer')
-
-        class Email(FancyValidator):
-            def _to_python(self, value):
-                if '@' not in value:
-                    raise TGValidationError('not email')
-                return value
-
-        class StringBool(FancyValidator):
-            def validate_python(self, value, state=None):
-                return value
-
-            def _to_python(self, value):
-                if not value:
-                    return value
-
-                if value == 'True':
-                    return True
-                raise TGValidationError('Not True')
-
-    class tw2c(object):
-        class IntValidator(validators.Int):
-            pass
 
 def setup():
     setup_session_dir()
@@ -119,6 +79,13 @@ class ColonLessGenericValidator(object):
     def validate(self, value, state=None):
         raise validators.Invalid('Unknown Error', value, {'_the_form':'Unknown Error'})
 
+def error_handler_function(controller_instance, uid, num):
+    return 'UID: %s' % uid
+
+class ErrorHandlerCallable(object):
+    def __call__(self, controller_instance, uid, num):
+        return 'UID: %s' % uid
+
 class BasicTGController(TGController):
     @expose()
     @validate(ColonLessGenericValidator())
@@ -210,20 +177,24 @@ class BasicTGController(TGController):
     def tw2_dict_validation(self, **kwargs):
         return str(tg.request.validation['errors'])
 
+    @expose('text/plain')
+    @validate(form=FormWithFieldSet, error_handler=tw2form_error_handler)
+    def tw2_fieldset_submit(self, **kwargs):
+        return 'passed validation'
+
     @expose()
     def set_lang(self, lang=None):
         tg.session['tg_lang'] = lang
         tg.session.save()
         return 'ok'
 
-    if not PY3:
-        @expose()
-        @validate(validators=Pwd())
-        def password(self, pwd1, pwd2):
-            if tg.request.validation['errors']:
-                return "There was an error"
-            else:
-                return "Password ok!"
+    @expose()
+    @validate(validators=Pwd())
+    def password(self, pwd1, pwd2):
+        if tg.request.validation['errors']:
+            return "There was an error"
+        else:
+            return "Password ok!"
 
     @expose('json:')
     @before_render(lambda rem,params,output:output.update({'GOT_ERROR':'HOOKED'}))
@@ -251,13 +222,88 @@ class BasicTGController(TGController):
     def validate_other_error_handler(self, some_int):
         return dict(response=some_int)
 
-class TestTGController(TestWSGIController):
+    def unexposed_error_handler(self, uid, **kw):
+        return 'UID: %s' % uid
 
+    @expose()
+    @validate({'uid': validators.Int(),
+               'num': validators.Int()},
+              error_handler=unexposed_error_handler)
+    def validate_unexposed(self, uid, num):
+        return 'HUH'
+
+    @expose()
+    @validate({'num': validators.Int()},
+              error_handler=partial(unexposed_error_handler,
+                                    uid=5))
+    def validate_partial(self, num):
+        return 'HUH'
+
+    @expose()
+    @validate({'uid': tw2c.IntValidator(),
+               'num': tw2c.IntValidator()},
+              error_handler=error_handler_function)
+    def validate_function(self, uid, num):
+        return 'HUH'
+
+    @expose()
+    @validate({'uid': validators.Int(),
+               'num': validators.Int()},
+              error_handler=ErrorHandlerCallable())
+    def validate_callable(self, uid, num):
+        return 'HUH'
+
+    @expose()
+    @before_call(lambda remainder, params: params.setdefault('num', 5))
+    def hooked_error_handler(self, uid, num):
+        return 'UID: %s, NUM: %s' % (uid, num)
+
+    @expose()
+    @validate({'uid': validators.Int()},
+              error_handler=hooked_error_handler)
+    def validate_hooked(self, uid):
+        return 'HUH'
+
+    @expose()
+    def manually_handle_validation(self):
+        # This is done to check that we don't break compatibility
+        # with external modules that perform custom validation like tgext.socketio
+
+        controller = self.__class__.validate_function
+        args = (2, 'NaN')
+        try:
+            output = ''
+            validate_params = get_params_with_argspec(controller, {}, args)
+            params = DecoratedController._perform_validate(controller,
+                                                           validate_params)
+        except validation_errors as inv:
+            handler, output = DecoratedController._handle_validation_errors(controller,
+                                                                            args, {},
+                                                                            inv, None)
+
+        return output
+
+def ControllerWrapperForErrorHandler(app_config, caller):
+    def call(*args, **kw):
+        value = caller(*args, **kw)
+        return value + 'X'
+    return call
+
+class TestTGController(TestWSGIController):
     def setUp(self):
         TestWSGIController.setUp(self)
         tg.config.update({
             'paths': {'root': data_dir},
-            'package': tests})
+            'package': tests,
+        })
+
+        # Mimic configuration of a controller wrapper, this is required as
+        # TestWSGIController doesn't actually create an AppConfig
+        # so configurations don't get resolved.
+        cwrapper = ControllerWrapperForErrorHandler(None, call_controller)
+        wrappers_conf = {default_im_func(BasicTGController.hooked_error_handler): cwrapper}
+        tg.config.update({'dedicated_controller_wrappers': wrappers_conf})
+
         self.app = make_app(BasicTGController)
 
     def test_basic_validation_and_jsonification(self):
@@ -358,8 +404,6 @@ class TestTGController(TestWSGIController):
             'Error message not found: %r' % values['errors']
 
     def test_tw2form_validation(self):
-        if PY3: raise SkipTest()
-
         form_values = {'title': 'Razer', 'year': "t007"}
         resp = self.app.post('/send_tw2_to_error_handler', form_values)
         values = loads(resp.body.decode('utf-8'))
@@ -367,8 +411,6 @@ class TestTGController(TestWSGIController):
         'Error message not found: %r' % values['errors']
 
     def test_tw2dict_validation(self):
-        if PY3: raise SkipTest()
-
         resp = self.app.post('/tw2_dict_validation', {'param': "7"})
         assert '{}' in str(resp.body)
 
@@ -401,8 +443,6 @@ class TestTGController(TestWSGIController):
 
     def test_form_validation_error(self):
         """Test schema validation"""
-        if PY3: raise SkipTest()
-
         form_values = {'pwd1': 'me', 'pwd2': 'you'}
         resp = self.app.post('/password', form_values)
         assert "There was an error" in resp, resp
@@ -426,4 +466,41 @@ class TestTGController(TestWSGIController):
 
     def test_validation_error_has_message(self):
         e = TGValidationError('This is a validation error')
-        assert str(e) == 'This is a validation error'
\ No newline at end of file
+        assert str(e) == 'This is a validation error'
+
+    def test_tw2_fieldset(self):
+        form_values = {'fields1:f1': 'Razer', 'fields2:f2': "t007"}
+        resp = self.app.post('/tw2_fieldset_submit', form_values)
+        values = loads(resp.body.decode('utf-8'))
+
+        assert "Must be an integer" in values['errors'].get('fields2:f2', ''),\
+        'Error message not found: %r' % values['errors']
+
+    def test_validate_partial(self):
+        resp = self.app.post('/validate_partial', {'num': 'NaN'})
+        assert resp.text == 'UID: 5', resp
+
+    def test_validate_unexposed(self):
+        resp = self.app.post('/validate_unexposed', {'uid': 2,
+                                                     'num': 'NaN'})
+        assert resp.text == 'UID: 2', resp
+
+    def test_validate_function(self):
+        resp = self.app.post('/validate_function', {'uid': 2,
+                                                    'num': 'NaN'})
+        assert resp.text == 'UID: 2', resp
+
+    def test_validate_callable(self):
+        resp = self.app.post('/validate_callable', {'uid': 2,
+                                                    'num': 'NaN'})
+        assert resp.text == 'UID: 2', resp
+
+    def test_validate_hooked(self):
+        resp = self.app.post('/validate_hooked', {'uid': 'NaN'})
+        assert resp.text == 'UID: NaN, NUM: 5X', resp
+
+    def test_manually_handle_validation(self):
+        # This is done to check that we don't break compatibility
+        # with external modules that perform custom validation like tgext.socketio
+        resp = self.app.post('/manually_handle_validation')
+        assert resp.text == 'UID: 2', resp
\ No newline at end of file
