@@ -8,12 +8,13 @@ from copy import copy, deepcopy
 import mimetypes
 from collections import MutableMapping as DictMixin, deque
 
-from tg.i18n import ugettext, ungettext, get_lang
+from tg.i18n import ugettext, get_lang
 
 from tg.support.middlewares import SessionMiddleware, CacheMiddleware
-from tg.support.middlewares import StaticsMiddleware, SeekableRequestBodyMiddleware, DBSessionRemoverMiddleware
+from tg.support.middlewares import StaticsMiddleware, SeekableRequestBodyMiddleware, \
+    DBSessionRemoverMiddleware
 from tg.support.registry import RegistryManager
-from paste.deploy.converters import asbool, asint
+from tg.support.converters import asbool, asint
 from tg.request_local import config as reqlocal_config
 
 import tg
@@ -22,6 +23,12 @@ from tg.util import Bunch, get_partial_dict, DottedFileNameFinder, call_controll
 from tg.configuration import milestones
 from tg.configuration.utils import TGConfigError
 
+from tg.renderers.genshi import GenshiRenderer
+from tg.renderers.json import JSONRenderer
+from tg.renderers.jinja import JinjaRenderer
+from tg.renderers.mako import MakoRenderer
+from tg.renderers.kajiki import KajikiRenderer
+
 log = logging.getLogger(__name__)
 
 
@@ -154,7 +161,10 @@ class AppConfig(Bunch):
         self.renderers = []
         self.default_renderer = 'genshi'
         self.render_functions = Bunch()
- 
+        self.rendering_engines = {}
+        self.rendering_engines_without_vars = set()
+        self.rendering_engines_options = {}
+
         self.enable_routes = False
         self.enable_routing_args = False
         self.disable_request_extensions = minimal
@@ -203,16 +213,11 @@ class AppConfig(Bunch):
         if root_controller is not None:
             self['tg.root_controller'] = root_controller
 
-    def __setitem__(self, key, value):
-        #some entries are required at application setup time
-        #so we need to keep them in sync between tg.config and AppConfig.
-
-        #This is especially true for available renderers which enable
-        #engine check in tg.decorators.Decorator.register_template_engine
-        if key in ['default_renderer', 'renderers']:
-            config[key] = value
-        return super(AppConfig, self).__setattr__(key, value)
-    __setattr__ = __setitem__
+        self.register_rendering_engine(JSONRenderer)
+        self.register_rendering_engine(GenshiRenderer)
+        self.register_rendering_engine(MakoRenderer)
+        self.register_rendering_engine(JinjaRenderer)
+        self.register_rendering_engine(KajikiRenderer)
 
     def get_root_module(self):
         root_module_path = self.paths['root']
@@ -267,17 +272,20 @@ class AppConfig(Bunch):
                                 'environment has already been loaded')
 
         self.application_wrappers_dependencies.setdefault(after, []).append(wrapper)
-        milestones.environment_loaded.register(self._order_wrappers)
+        milestones.environment_loaded.register(self._configure_application_wrappers)
 
-    def _order_wrappers(self):
-        visit_queue = deque([False, None])
-        while visit_queue:
-            current = visit_queue.popleft()
-            if current not in (False, None):
-                self.application_wrappers.append(current)
+    def register_rendering_engine(self, factory):
+        """Registers a rendering engine ``factory``.
 
-            dependant_wrappers = self.application_wrappers_dependencies.pop(current, [])
-            visit_queue.extendleft(reversed(dependant_wrappers))
+        Rendering engine factories are :class:`tg.renderers.base.RendererFactory`
+        subclasses in charge of creating a rendering engine.
+
+        """
+        for engine, options in factory.engines.items():
+            self.rendering_engines[engine] = factory
+            self.rendering_engines_options[engine] = options
+            if factory.with_tg_vars is False:
+                self.rendering_engines_without_vars.add(engine)
 
     def _setup_startup_and_shutdown(self):
         for cmd in self.call_on_startup:
@@ -295,7 +303,17 @@ class AppConfig(Bunch):
             else:
                 log.warn("Unable to register %s for shutdown" % cmd )
 
-    def _setup_package_paths(self):
+    def _configure_application_wrappers(self):
+        visit_queue = deque([False, None])
+        while visit_queue:
+            current = visit_queue.popleft()
+            if current not in (False, None):
+                self.application_wrappers.append(current)
+
+            dependant_wrappers = self.application_wrappers_dependencies.pop(current, [])
+            visit_queue.extendleft(reversed(dependant_wrappers))
+
+    def _configure_package_paths(self):
         root = os.path.dirname(os.path.abspath(self.package.__file__))
         # The default paths:
         paths = Bunch(root=root,
@@ -315,7 +333,7 @@ class AppConfig(Bunch):
 
         """
         # Load the mimetypes with its default types
-        mimetypes.init()
+        self.mimetypes = mimetypes.MimeTypes()
 
         try:
             self.package_name = self.package.__name__
@@ -346,15 +364,36 @@ class AppConfig(Bunch):
         errorware['debug'] = conf['debug']
         if not errorware['debug']:
             errorware['debug'] = False
-            errorware['error_email'] = conf.get('email_to')
-            errorware['error_log'] = conf.get('error_log', None)
-            errorware['smtp_server'] = conf.get('smtp_server', 'localhost')
-            errorware['smtp_use_tls'] = asbool(conf.get('smtp_use_tls', False))
-            errorware['smtp_username'] = conf.get('smtp_username')
-            errorware['smtp_password'] = conf.get('smtp_password')
-            errorware['error_subject_prefix'] = conf.get('error_subject_prefix', 'WebApp Error: ')
-            errorware['from_address'] = conf.get('from_address', conf.get('error_email_from', 'turbogears@yourapp.com'))
-            errorware['error_message'] = conf.get('error_message', 'An internal server error occurred')
+
+            trace_errors_config = coerce_config(conf, 'trace_errors.', {'smtp_use_tls': asbool,
+                                                                        'dump_request_size': asint,
+                                                                        'dump_request': asbool,
+                                                                        'dump_local_frames': asbool,
+                                                                        'dump_local_frames_count': asint})
+            if not trace_errors_config:
+                # backward compatibility
+                warnings.warn("direct usage of error tracing options has been deprecated, "
+                              "please specify them as trace_errors.option_name instad of directly "
+                              "setting option_name. EXAMPLE: trace_errors.error_email", DeprecationWarning)
+
+                trace_errors_config['error_email'] = conf.get('email_to')
+                trace_errors_config['error_log'] = conf.get('error_log', None)
+                trace_errors_config['smtp_server'] = conf.get('smtp_server', 'localhost')
+                trace_errors_config['smtp_use_tls'] = asbool(conf.get('smtp_use_tls', False))
+                trace_errors_config['smtp_username'] = conf.get('smtp_username')
+                trace_errors_config['smtp_password'] = conf.get('smtp_password')
+                trace_errors_config['error_subject_prefix'] = conf.get('error_subject_prefix', 'WebApp Error: ')
+                trace_errors_config['from_address'] = conf.get('from_address', conf.get('error_email_from', 'turbogears@yourapp.com'))
+                trace_errors_config['error_message'] = conf.get('error_message', 'An internal server error occurred')
+            else:
+                # Provide Defaults
+                trace_errors_config.setdefault('error_subject_prefix',
+                                               'WebApp Error: ')
+                trace_errors_config.setdefault('error_message',
+                                               'An internal server error occurred')
+
+            errorware.update(trace_errors_config)
+
         conf['tg.errorware'] = errorware
 
         # Copy in some defaults
@@ -397,6 +436,8 @@ class AppConfig(Bunch):
             config['tg.strict_tmpl_context'] = False
 
         self.after_init_config()
+        self._configure_mimetypes()
+
         milestones.config_ready.reach()
 
     def _configure_renderers(self):
@@ -409,6 +450,14 @@ class AppConfig(Bunch):
             log.warn('Default renderer not in renders, automatically switching to %s' % first_renderer)
             self.default_renderer = first_renderer
 
+    def _configure_mimetypes(self):
+        lookup = {'.json': 'application/json',
+                  '.js': 'application/javascript'}
+        lookup.update(config.get('mimetype_lookup', {}))
+
+        for key, value in lookup.items():
+            self.mimetypes.add_type(value, key)
+
     def after_init_config(self):
         """
         Override this method to set up configuration variables at the application
@@ -465,7 +514,7 @@ class AppConfig(Bunch):
         from routes import Mapper
 
         map = Mapper(directory=config['paths']['controllers'],
-                    always_scan=config['debug'])
+                     always_scan=config['debug'])
 
         # Setup a default route for the root of object dispatch
         map.connect('*url', controller='root', action='routes_placeholder')
@@ -493,216 +542,6 @@ class AppConfig(Bunch):
         if config.get('tg.pylons_compatible', True):
             config['pylons.app_globals'] = g
 
-    def setup_mako_renderer(self, use_dotted_templatenames=None):
-        """Setup a renderer and loader for mako templates.
-
-        Override this to customize the way that the mako template
-        renderer is setup.  In particular if you want to setup
-        a different set of search paths, different encodings, or
-        additonal imports, all you need to do is update the
-        ``TemplateLookup`` constructor.
-
-        You can also use your own render_mako function instead of the one
-        provided by tg.render.
-
-        """
-
-        from tg.render import render_mako
-
-        if not use_dotted_templatenames:
-            use_dotted_templatenames = asbool(config.get('use_dotted_templatenames', 'true'))
-
-
-        # If no dotted names support was required we will just setup
-        # a file system based template lookup mechanism.
-        compiled_dir = tg.config.get('templating.mako.compiled_templates_dir', None)
-
-        if not compiled_dir or compiled_dir.lower() in ('none', 'false'):
-            # Cache compiled templates in-memory
-            compiled_dir = None
-        else:
-            bad_path = None
-            if os.path.exists(compiled_dir):
-                if not os.access(compiled_dir, os.W_OK):
-                    bad_path = compiled_dir
-                    compiled_dir = None
-            else:
-                try:
-                    os.makedirs(compiled_dir)
-                except:
-                    bad_path = compiled_dir
-                    compiled_dir = None
-            if bad_path:
-                log.warn("Unable to write cached templates to %r; falling back "
-                         "to an in-memory cache. Please set the `templating.mak"
-                         "o.compiled_templates_dir` configuration option to a "
-                         "writable directory." % bad_path)
-
-        if use_dotted_templatenames:
-            # Support dotted names by injecting a slightly different template
-            # lookup system that will return templates from dotted template notation.
-            from tg.dottednames.mako_lookup import DottedTemplateLookup
-            config['tg.app_globals'].mako_lookup = DottedTemplateLookup(
-                input_encoding='utf-8', output_encoding='utf-8',
-                imports=['from markupsafe import escape_silent as escape'],
-                module_directory=compiled_dir,
-                default_filters=['escape'],
-                auto_reload_templates=self.auto_reload_templates)
-
-        else:
-            from mako.lookup import TemplateLookup
-            config['tg.app_globals'].mako_lookup = TemplateLookup(
-                directories=self.paths['templates'],
-                module_directory=compiled_dir,
-                input_encoding='utf-8', output_encoding='utf-8',
-                imports=['from markupsafe import escape_silent as escape'],
-                default_filters=['escape'],
-                filesystem_checks=self.auto_reload_templates)
-
-        self.render_functions.mako = render_mako
-
-    def setup_chameleon_genshi_renderer(self): #pragma: no cover
-        """Setup a renderer and loader for the chameleon.genshi engine."""
-        from tg.render import RenderChameleonGenshi
-
-        try:
-            import chameleon.genshi.loader
-        except ImportError:
-            return False
-
-        if config.get('use_dotted_templatenames', True):
-            from tg.dottednames.chameleon_genshi_lookup \
-                import ChameleonGenshiTemplateLoader as TemplateLoader
-        else:
-            from chameleon.genshi.loader import TemplateLoader
-        loader = TemplateLoader(search_path=self.paths.templates,
-                                auto_reload=self.auto_reload_templates)
-
-        self.render_functions.chameleon_genshi = RenderChameleonGenshi(loader)
-
-    def setup_genshi_renderer(self):
-        """Setup a renderer and loader for Genshi templates.
-
-        Override this to customize the way that the internationalization
-        filter, template loader
-
-        """
-        from tg.render import RenderGenshi
-        from genshi.filters import Translator
-
-        def template_loaded(template):
-            """Plug-in our i18n function to Genshi, once the template is loaded.
-
-            This function will be called by the Genshi TemplateLoader after
-            loading the template.
-
-            """
-            translator = Translator(ugettext)
-            template.filters.insert(0, translator)
-            if hasattr(template, 'add_directives'):
-                template.add_directives(Translator.NAMESPACE, translator)
-
-        if config.get('use_dotted_templatenames', True):
-            from tg.dottednames.genshi_lookup \
-                import GenshiTemplateLoader as TemplateLoader
-        else:
-            from genshi.template import TemplateLoader
-        loader = TemplateLoader(search_path=self.paths.templates,
-                                max_cache_size=asint(self.get('genshi.max_cache_size', 30)),
-                                auto_reload=self.auto_reload_templates,
-                                callback=template_loaded)
-
-        self.render_functions.genshi = RenderGenshi(loader)
-
-    def setup_kajiki_renderer(self):
-        """Setup a renderer and loader for the fastpt engine."""
-        from tg.dottednames.kajiki_lookup import KajikiTemplateLoader
-        from tg.render import render_kajiki
-        loader = KajikiTemplateLoader(self.paths.templates[0],
-                                      force_mode='xml',
-                                      reload=self.auto_reload_templates)
-        config['tg.app_globals'].kajiki_loader = loader
-        self.render_functions.kajiki = render_kajiki
-
-    def setup_jinja_renderer(self):
-        """Setup a renderer and loader for Jinja2 templates."""
-        from jinja2 import ChoiceLoader, Environment
-        from jinja2.filters import FILTERS
-        from tg.render import render_jinja
-
-        if config.get('use_dotted_templatenames', True):
-            from tg.dottednames.jinja_lookup import JinjaTemplateLoader as TemplateLoader
-        else:
-            from jinja2 import FileSystemLoader as TemplateLoader
-
-        if not 'jinja_extensions' in self :
-            self.jinja_extensions = []
-
-        # Add i18n extension by default
-        if not "jinja2.ext.i18n" in self.jinja_extensions:
-            self.jinja_extensions.append("jinja2.ext.i18n")
-
-        if not 'jinja_filters' in self:
-            self.jinja_filters = {}
-
-        loader = ChoiceLoader(
-            [TemplateLoader(path) for path in self.paths['templates']])
-
-        config['tg.app_globals'].jinja2_env = Environment(loader=loader, autoescape=True,
-                 auto_reload=self.auto_reload_templates, extensions=self.jinja_extensions)
-
-        # Try to load custom filters module under app_package.lib.templatetools
-        try:
-            if not self.package_name:
-                raise AttributeError()
-
-            filter_package = self.package_name + ".lib.templatetools"
-            autoload_lib = __import__(filter_package, {}, {}, ['jinja_filters'])
-            try:
-                autoload_filters = dict(
-                    map(lambda x: (x,autoload_lib.jinja_filters.__dict__[x]), autoload_lib.jinja_filters.__all__)
-                )
-            except AttributeError: #pragma: no cover
-                autoload_filters = dict(
-                    filter(lambda x: callable(x[1]),
-                        autoload_lib.jinja_filters.__dict__.iteritems())
-                )
-        except (ImportError, AttributeError):
-            autoload_filters = {}
-
-        # Add jinja filters
-        filters = dict(FILTERS, **autoload_filters)
-        filters.update(self.jinja_filters)
-        config['tg.app_globals'].jinja2_env.filters = filters
-
-        # Jinja's unable to request c's attributes without strict_c
-        config['tg.strict_tmpl_context'] = True
-
-        # Add gettext functions to the jinja environment
-        config['tg.app_globals'].jinja2_env.install_gettext_callables(ugettext, ungettext)
-
-        self.render_functions.jinja = render_jinja
-
-    def setup_amf_renderer(self): #pragma: no cover
-        try:
-            from tg.amfify import render_amf
-        except ImportError:
-            log.error('PyAMF not installed')
-            return False
-
-        self.render_functions.amf = render_amf
-
-    def setup_json_renderer(self):
-        from tg.render import render_json
-        self.render_functions.json = render_json
-
-    def setup_mimetypes(self):
-        lookup = {'.json':'application/json'}
-        lookup.update(config.get('mimetype_lookup', {}))
-
-        for key, value in lookup.items():
-            mimetypes.add_type(value, key)
-
     def setup_persistence(self):
         """Override this method to define how your application configures it's persistence model.
            the default is to setup sqlalchemy from the cofiguration file, but you might choose
@@ -815,6 +654,12 @@ class AppConfig(Bunch):
         config['tg.app_globals'].sa_engine = engine
         self.package.model.init_model(engine)
 
+        if not hasattr(self, 'DBSession'):
+            # If the user hasn't specified a scoped_session, assume
+            # he/she uses the default DBSession in model
+            model = getattr(self, 'model', self.package.model)
+            self.DBSession = model.DBSession
+
     def setup_auth(self):
         """
         Override this method to define how you would like the authentication options
@@ -860,11 +705,25 @@ class AppConfig(Bunch):
     def _setup_renderers(self):
         for renderer in self.renderers[:]:
             setup = getattr(self, 'setup_%s_renderer'%renderer, None)
-            if setup:
+            if setup is not None:
+                # Backward compatible old-way of configuring rendering engines
+                warnings.warn("Using setup_NAME_renderer to configure rendering engines"
+                              "is now deprecated, please use register_rendering_engine "
+                              "with a tg.renderers.base.RendererFactory subclass instead",
+                              DeprecationWarning, stacklevel=2)
+
                 success = setup()
                 if success is False:
                     log.error('Failed to initialize %s template engine, removing it...' % renderer)
                     self.renderers.remove(renderer)
+            elif renderer in self.rendering_engines:
+                rendering_engine = self.rendering_engines[renderer]
+                engines = rendering_engine.create(config, config['tg.app_globals'])
+                if engines is None:
+                    log.error('Failed to initialize %s template engine, removing it...' % renderer)
+                    self.renderers.remove(renderer)
+                else:
+                    self.render_functions.update(engines)
             else:
                 raise TGConfigError('This configuration object does not support the %s renderer' % renderer)
 
@@ -882,8 +741,8 @@ class AppConfig(Bunch):
 
         def load_environment(global_conf, app_conf):
             """Configure the TurboGears environment via ``tg.configuration.config``."""
-            global_conf=Bunch(global_conf)
-            app_conf=Bunch(app_conf)
+            global_conf = Bunch(global_conf)
+            app_conf = Bunch(app_conf)
 
             try:
                 app_package = self.package
@@ -894,7 +753,7 @@ class AppConfig(Bunch):
                 app_package = None
 
             if app_package:
-                self._setup_package_paths()
+                self._configure_package_paths()
 
             self._init_config(global_conf, app_conf)
 
@@ -907,11 +766,12 @@ class AppConfig(Bunch):
             if app_package:
                 self.setup_helpers_and_globals()
 
-            self.setup_mimetypes()
             self.setup_auth()
             self._setup_renderers()
             self.setup_persistence()
 
+            # Trigger milestone here so that it gets triggered even when
+            # websetup (setup-app command) is performed.
             milestones.environment_loaded.reach()
 
         return load_environment
@@ -1128,11 +988,24 @@ class AppConfig(Bunch):
         """
         from tw2.core.middleware import Config, TwMiddleware
 
-        tw2_engines = [self.default_renderer] + Config.preferred_rendering_engines
-        default_tw2_config = dict( default_engine=self.default_renderer,
+        shared_engines = list(set(self.renderers) & set(Config.preferred_rendering_engines))
+        if not shared_engines:
+            raise TGConfigError('None of the configured rendering engines is supported'
+                                'by ToscaWidgets2, unable to configure ToscaWidgets.')
+
+        if self.default_renderer in shared_engines:
+            tw2_engines = [self.default_renderer] + shared_engines
+            tw2_default_engine = self.default_renderer
+        else:
+            # If preferred rendering engine is not available in TW2, fallback to another one
+            # This happens for Kajiki which is not supported by recent TW2 versions.
+            tw2_engines = shared_engines
+            tw2_default_engine = shared_engines[0]
+
+        default_tw2_config = dict( default_engine=tw2_default_engine,
                                    preferred_rendering_engines=tw2_engines,
                                    translator=ugettext,
-                                   get_lang=get_lang,
+                                   get_lang=lambda: get_lang(all=False),
                                    auto_reload_templates=self.auto_reload_templates,
                                    controller_prefix='/tw2/controllers/',
                                    res_prefix='/tw2/resources/',
@@ -1173,10 +1046,10 @@ class AppConfig(Bunch):
 
     def add_ming_middleware(self, app):
         """Set up the ming middleware for the unit of work"""
-        import ming.orm.middleware
-        return ming.orm.middleware.MingMiddleware(app)
+        import ming.odm.middleware
+        return ming.odm.middleware.MingMiddleware(app)
 
-    def add_dbsession_remover_middleware(self, app):
+    def add_sqlalchemy_middleware(self, app):
         """Set up middleware that cleans up the sqlalchemy session.
 
         The default behavior of TG 2 is to clean up the session on every
@@ -1232,6 +1105,10 @@ class AppConfig(Bunch):
             if load_environment:
                 load_environment(global_conf, app_conf)
 
+            # trigger the environment_loaded milestone again, so that
+            # when load_environment is not provided the attached actions gets performed anyway.
+            milestones.environment_loaded.reach()
+
             # Apply controller wrappers to controller caller
             self._setup_controller_wrappers()
 
@@ -1273,11 +1150,7 @@ class AppConfig(Bunch):
             # a string needs to be applied before this point.
 
             if self.use_sqlalchemy:
-                if not hasattr(self, 'DBSession'):
-                    # If the user hasn't specified a scoped_session, assume
-                    # he/she uses the default DBSession in model
-                    self.DBSession = self.model.DBSession
-                app = self.add_dbsession_remover_middleware(app)
+                app = self.add_sqlalchemy_middleware(app)
 
             if self.use_ming:
                 app = self.add_ming_middleware(app)
